package com.tzld.longarticle.recommend.server.service.filter.strategy;

import com.tzld.longarticle.recommend.server.common.ThreadPoolFactory;
import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.remote.ArticleListRemoteService;
import com.tzld.longarticle.recommend.server.repository.entity.crawler.Article;
import com.tzld.longarticle.recommend.server.service.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.filter.FilterParam;
import com.tzld.longarticle.recommend.server.service.filter.FilterResult;
import com.tzld.longarticle.recommend.server.service.filter.FilterStrategy;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
@Component
@Slf4j
public class HistoryTitleStrategy implements FilterStrategy {

    @Autowired
    private ArticleListRemoteService articleListRemoteService;
    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;

    private static final List<Integer> firstSecondIndex = Arrays.asList(1, 2);
    private static final List<Integer> allIndex = Arrays.asList(1, 2, 3, 4, 5, 6, 7, 8);

    private final ExecutorService pool = ThreadPoolFactory.deDuplicatePool();

    @Override
    public FilterResult filter(FilterParam param) {
        long start = System.currentTimeMillis();
        FilterResult filterResult = new FilterResult();
        List<String> result = new ArrayList<>();
        List<Content> filterContents = new ArrayList<>();
        List<Article> allArticleList = articleListRemoteService.articleList(param.getAccountName(), allIndex);
        List<String> allTitleList = allArticleList.stream().map(Article::getTitle).distinct().collect(Collectors.toList());
        List<Article> firstSecondArticleList = allArticleList.stream().filter(article -> firstSecondIndex.contains(article.getItemIndex())).collect(Collectors.toList());
        List<String> firstSecondTitleList = firstSecondArticleList.stream().map(Article::getTitle).distinct().collect(Collectors.toList());

        List<String> firstSecondContentPool = new ArrayList<>();
        String[] contentPoolConfig = accountContentPoolConfigService.getContentPools(param.getAccountName());
        if (Objects.nonNull(contentPoolConfig)) {
            firstSecondContentPool.addAll(Arrays.asList(contentPoolConfig[0], contentPoolConfig[1]));
        }
        List<Future<Content>> futures = new ArrayList<>();
        CountDownLatch cdl = new CountDownLatch(param.getContents().size());
        for (Content content : param.getContents()) {
            Future<Content> future = pool.submit(() -> {
                try {
                    boolean isDuplicate;
                    if (CollectionUtils.isNotEmpty(firstSecondContentPool) && firstSecondContentPool.contains(content.getContentPoolType())) {
                        // 四个内容池 配置 判断头条，次头条
                        isDuplicate = TitleSimilarCheckUtil.isDuplicateContent(content.getTitle(), firstSecondTitleList);
                    } else {
                        isDuplicate = TitleSimilarCheckUtil.isDuplicateContent(content.getTitle(), allTitleList);
                    }
                    if (isDuplicate) {
                        content.setFilterReason("历史已发布文章");
                    }
                    return content;
                } finally {
                    cdl.countDown();
                }
            });
            futures.add(future);
        }
        try {
            cdl.await();
        } catch (InterruptedException e) {
            log.error("filter error", e);
            return null;
        }

        for (Future<Content> f : futures) {
            try {
                Content content = f.get();
                if (StringUtils.hasText(content.getFilterReason())) {
                    filterContents.add(content);
                } else {
                    result.add(content.getId());
                }
            } catch (Exception e) {
                log.error("future get error ", e);
            }
        }
        filterResult.setContentIds(result);
        filterResult.setFilterContent(filterContents);
        log.info("HistoryTitleStrategy cost:{}", System.currentTimeMillis() - start);
        return filterResult;
    }

}
